'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var rollupPluginCommonjs = _interopDefault(require('rollup-plugin-commonjs'));
var rollupPluginJson = _interopDefault(require('rollup-plugin-json'));
var path = _interopDefault(require('path'));
var rollupPluginReplace = _interopDefault(require('rollup-plugin-replace'));
var rollupPluginNodeResolve = _interopDefault(require('rollup-plugin-node-resolve'));
var rollup$1 = _interopDefault(require('rollup'));

const { rollup } = rollup$1;

const ROLLUP_INPUT_DEFAULTS = {
  // Only bundle local package files
  external: (id) => /^[^./]/.test(id),
  plugins: [
    rollupPluginReplace({
      'process.env.NODE_ENV': `"${process.env.NODE_ENV}"` || '"development"'
    }),
    rollupPluginNodeResolve({
      browser: true
    }),
    rollupPluginJson(),
    rollupPluginCommonjs({
      sourceMap: false
    })
  ],
  treeshake: false
};
const ROLLUP_OUTPUT_DEFAULTS = {
  format: 'es',
  sourcemap: false
};

const { resolveId: resolveMainModule } = ROLLUP_INPUT_DEFAULTS.plugins[1];

/**
 * Bundle package at 'id'
 *
 * @param { string } id
 * @param { string } outputpath
 * @param { object } overrideOptions
 * @param { (err: Error) => void } fn
 */
var bundleWorker = async function bundle(id, outputpath, overrideOptions, fn) {
  overrideOptions = parseOptions(overrideOptions);
  const input = await resolveMainModule(id, path.resolve('index.js'));
  const inputOptions = {
    input,
    ...ROLLUP_INPUT_DEFAULTS,
    ...overrideOptions.input
  };
  const outputOptions = {
    file: outputpath,
    ...ROLLUP_OUTPUT_DEFAULTS,
    ...overrideOptions.output
  };

  try {
    const bundled = await rollup(inputOptions);
    await bundled.write(outputOptions);
    fn();
  } catch (err) {
    console.log(err);
    fn(err);
  }
};

/**
 * Parse override options
 *
 * @param { object } options
 * @returns { object }
 */
function parseOptions(options) {
  if (!options) {
    return { input: {}, output: {} };
  }

  const { input, treeshake, output = {}, watch, ...inputOverride } = options;
  const { file, format, sourcemap, ...outputOverride } = output;

  return { input: inputOverride, output: outputOverride };
}

module.exports = bundleWorker;
